@@ -16,12 +16,15 @@
 
 """Serializer object for CHK based inventory storage."""
 
-from __future__ import absolute_import
+from io import (
+    BytesIO,
+    )
 
 from .. import lazy_import
 lazy_import.lazy_import(globals(),
                         """
 from breezy.bzr import (
+    serializer,
     xml_serializer,
     )
 """)
@@ -34,15 +37,12 @@ from .. import (
 from . import (
     serializer,
     )
-from ..sixish import (
-    BytesIO,
-    )
 
 
 def _validate_properties(props, _decode=cache_utf8._utf8_decode):
     # TODO: we really want an 'isascii' check for key
     # Cast the utf8 properties into Unicode 'in place'
-    return {_decode(key)[0]: _decode(value)[0] for key, value in props.items()}
+    return {_decode(key)[0]: _decode(value, 'surrogateescape')[0] for key, value in props.items()}
 
 
 def _is_format_10(value):
@@ -90,7 +90,7 @@ class BEncodeRevisionSerializer1(object):
         # which changes infrequently.
         revprops = {}
         for key, value in rev.properties.items():
-            revprops[encode_utf8(key)[0]] = encode_utf8(value)[0]
+            revprops[encode_utf8(key)[0]] = encode_utf8(value, 'surrogateescape')[0]
         ret.append((b'properties', revprops))
         ret.extend([
             (b"timestamp", b"%.3f" % rev.timestamp),
@@ -183,7 +183,7 @@ class CHKSerializer(serializer.Serializer):
                 entry_cache=entry_cache,
                 return_from_cache=return_from_cache)
         except xml_serializer.ParseError as e:
-            raise errors.UnexpectedInventoryFormat(e)
+            raise serializer.UnexpectedInventoryFormat(e)
 
     def read_inventory(self, f, revision_id=None):
         """Read an inventory from a file-like object."""
@@ -194,7 +194,7 @@ class CHKSerializer(serializer.Serializer):
             finally:
                 f.close()
         except xml_serializer.ParseError as e:
-            raise errors.UnexpectedInventoryFormat(e)
+            raise serializer.UnexpectedInventoryFormat(e)
 
     def write_inventory_to_lines(self, inv):
         """Return a list of lines with the encoded inventory."""
