@@ -16,8 +16,6 @@
 
 """InterRepository operations."""
 
-from __future__ import absolute_import
-
 from io import BytesIO
 import itertools
 
@@ -52,10 +50,6 @@ from ..repository import (
 from ..revision import (
     NULL_REVISION,
     )
-from ..sixish import (
-    viewitems,
-    viewvalues,
-    )
 from .. import (
     config,
     trace,
@@ -90,6 +84,7 @@ from .repository import (
     )
 from .remote import (
     RemoteGitRepository,
+    RemoteGitError,
     )
 from .unpeel_map import (
     UnpeelMap,
@@ -229,7 +224,7 @@ class InterToLocalGitRepository(InterToGitRepository):
                         new_stop_revids.append(revid)
                 stop_revids = set()
                 parent_map = graph.get_parent_map(new_stop_revids)
-                for parent_revids in viewvalues(parent_map):
+                for parent_revids in parent_map.values():
                     stop_revids.update(parent_revids)
                 pb.update("determining revisions to fetch", len(missing))
         return graph.iter_topo_order(missing)
@@ -248,7 +243,7 @@ class InterToLocalGitRepository(InterToGitRepository):
                 # broken symref?
                 continue
             revid = None
-            if not v.startswith(SYMREF):
+            if v and not v.startswith(SYMREF):
                 try:
                     for (kind, type_data) in self.source_store.lookup_git_sha(
                             v):
@@ -267,12 +262,12 @@ class InterToLocalGitRepository(InterToGitRepository):
         with self.source_store.lock_read():
             old_refs = self._get_target_bzr_refs()
             new_refs = update_refs(old_refs)
-            revidmap = self.fetch_objects(
+            revidmap = self.fetch_revs(
                 [(git_sha, bzr_revid)
                  for (git_sha, bzr_revid) in new_refs.values()
                  if git_sha is None or not git_sha.startswith(SYMREF)],
                 lossy=lossy)
-            for name, (gitid, revid) in viewitems(new_refs):
+            for name, (gitid, revid) in new_refs.items():
                 if gitid is None:
                     try:
                         gitid = revidmap[revid][0]
@@ -291,7 +286,7 @@ class InterToLocalGitRepository(InterToGitRepository):
                     result_refs[name] = (gitid, revid if not lossy else self.mapping.revision_id_foreign_to_bzr(gitid))
         return revidmap, old_refs, result_refs
 
-    def fetch_objects(self, revs, lossy, limit=None):
+    def fetch_revs(self, revs, lossy, limit=None):
         if not lossy and not self.mapping.roundtripping:
             for git_sha, bzr_revid in revs:
                 if (bzr_revid is not None and
@@ -314,8 +309,7 @@ class InterToLocalGitRepository(InterToGitRepository):
                         try:
                             self.mapping.revision_id_bzr_to_foreign(old_revid)
                         except InvalidRevisionId:
-                            refname = self.mapping.revid_as_refname(old_revid)
-                            self.target_refs[refname] = git_sha
+                            pass
                     revidmap[old_revid] = (git_sha, new_revid)
                 self.target_store.add_objects(object_generator)
                 return revidmap
@@ -338,7 +332,7 @@ class InterToLocalGitRepository(InterToGitRepository):
                               for revid in self.source.all_revision_ids()]
         self._warn_slow()
         try:
-            revidmap = self.fetch_objects(stop_revisions, lossy=lossy)
+            revidmap = self.fetch_revs(stop_revisions, lossy=lossy)
         except NoPushSupport:
             raise NoRoundtrippingSupport(self.source, self.target)
         return FetchResult(revidmap)
@@ -362,9 +356,9 @@ class InterToRemoteGitRepository(InterToGitRepository):
         def git_update_refs(old_refs):
             ret = {}
             self.old_refs = {
-                k: (v, None) for (k, v) in viewitems(old_refs)}
+                k: (v, None) for (k, v) in old_refs.items()}
             new_refs = update_refs(self.old_refs)
-            for name, (gitid, revid) in viewitems(new_refs):
+            for name, (gitid, revid) in new_refs.items():
                 if gitid is None:
                     git_sha = self.source_store._lookup_revision_sha1(revid)
                     gitid = unpeel_map.re_unpeel_tag(
@@ -377,8 +371,16 @@ class InterToRemoteGitRepository(InterToGitRepository):
             return ret
         self._warn_slow()
         with self.source_store.lock_read():
-            new_refs = self.target.send_pack(
+            result = self.target.send_pack(
                 git_update_refs, self.source_store.generate_lossy_pack_data)
+            if result is not None and not isinstance(result, dict):
+                for ref, error in result.ref_status.items():
+                    if error:
+                        raise RemoteGitError(
+                            'unable to update ref %r: %s' % (ref, error))
+                new_refs = result.refs
+            else:  # dulwich < 0.20.3
+                new_refs = result
         # FIXME: revidmap?
         return revidmap, self.old_refs, new_refs
 
@@ -415,12 +417,12 @@ class InterFromGitRepository(InterRepository):
 
         def determine_wants(refs):
             unpeel_lookup = {}
-            for k, v in viewitems(refs):
+            for k, v in refs.items():
                 if k.endswith(ANNOTATED_TAG_SUFFIX):
                     unpeel_lookup[v] = refs[k[:-len(ANNOTATED_TAG_SUFFIX)]]
             potential = set([unpeel_lookup.get(w, w) for w in wants])
             if include_tags:
-                for k, sha in viewitems(refs):
+                for k, sha in refs.items():
                     if k.endswith(ANNOTATED_TAG_SUFFIX):
                         continue
                     try:
@@ -463,8 +465,8 @@ class InterFromGitRepository(InterRepository):
             if if_present_ids is not None:
                 todo.update(if_present_ids)
         result_set = todo.difference(self.target.all_revision_ids())
-        result_parents = set(itertools.chain.from_iterable(viewvalues(
-            self.source.get_graph().get_parent_map(result_set))))
+        result_parents = set(itertools.chain.from_iterable(
+            self.source.get_graph().get_parent_map(result_set).values()))
         included_keys = result_set.intersection(result_parents)
         start_keys = result_set.difference(included_keys)
         exclude_keys = result_parents.difference(result_set)
@@ -489,7 +491,7 @@ class InterGitNonGitRepository(InterFromGitRepository):
 
     def determine_wants_all(self, refs):
         potential = set()
-        for k, v in viewitems(refs):
+        for k, v in refs.items():
             # For non-git target repositories, only worry about peeled
             if v == ZERO_SHA:
                 continue
@@ -563,7 +565,7 @@ class InterRemoteGitNonGitRepository(InterGitNonGitRepository):
         all_revs = self.target.all_revision_ids()
         parent_map = self.target.get_parent_map(all_revs)
         all_parents = set()
-        for values in viewvalues(parent_map):
+        for values in parent_map.values():
             all_parents.update(values)
         return set(all_revs) - all_parents
 
@@ -647,12 +649,12 @@ class InterGitGitRepository(InterFromGitRepository):
 
         def determine_wants(heads):
             old_refs = dict([(k, (v, None))
-                             for (k, v) in viewitems(heads.as_dict())])
+                             for (k, v) in heads.items()])
             new_refs = update_refs(old_refs)
             ref_changes.update(new_refs)
-            return [sha1 for (sha1, bzr_revid) in viewvalues(new_refs)]
+            return [sha1 for (sha1, bzr_revid) in new_refs.values()]
         self.fetch_objects(determine_wants, lossy=lossy)
-        for k, (git_sha, bzr_revid) in viewitems(ref_changes):
+        for k, (git_sha, bzr_revid) in ref_changes.items():
             self.target._git.refs[k] = git_sha
         new_refs = self.target.controldir.get_refs_container()
         return None, old_refs, new_refs
@@ -706,7 +708,7 @@ class InterGitGitRepository(InterFromGitRepository):
     def get_determine_wants_branches(self, branches, include_tags=False):
         def determine_wants(refs):
             ret = []
-            for name, value in viewitems(refs):
+            for name, value in refs.items():
                 if value == ZERO_SHA:
                     continue
 
@@ -786,3 +788,36 @@ class InterRemoteGitLocalGitRepository(InterGitGitRepository):
         """Be compatible with GitRepository."""
         return (isinstance(source, RemoteGitRepository) and
                 isinstance(target, LocalGitRepository))
+
+
+
+class InterLocalGitRemoteGitRepository(InterToGitRepository):
+
+    def fetch_refs(self, update_refs, lossy=False, overwrite=False):
+        """Import the gist of the ancestry of a particular revision."""
+        if lossy:
+            raise LossyPushToSameVCS(self.source, self.target)
+
+        def git_update_refs(old_refs):
+            ret = {}
+            self.old_refs = {
+                k: (v, None) for (k, v) in old_refs.items()}
+            new_refs = update_refs(self.old_refs)
+            for name, (gitid, revid) in new_refs.items():
+                if gitid is None:
+                    gitid = self.source_store._lookup_revision_sha1(revid)
+                if not overwrite:
+                    if remote_divergence(
+                            old_refs.get(name), gitid, self.source_store):
+                        raise DivergedBranches(self.source, self.target)
+                ret[name] = gitid
+            return ret
+        new_refs = self.target.send_pack(
+            git_update_refs,
+            self.source._git.generate_pack_data)
+        return None, self.old_refs, new_refs
+
+    @staticmethod
+    def is_compatible(source, target):
+        return (isinstance(source, LocalGitRepository) and
+                isinstance(target, RemoteGitRepository))
