@@ -18,8 +18,6 @@
 
 """Converters, etc for going between Bazaar and Git ids."""
 
-from __future__ import absolute_import
-
 import base64
 import stat
 
@@ -39,11 +37,6 @@ from ..revision import (
     NULL_REVISION,
     Revision,
     )
-from ..sixish import (
-    PY3,
-    text_type,
-    viewitems,
-    )
 from .errors import (
     NoPushSupport,
     )
@@ -121,6 +114,8 @@ def fix_person_identifier(text):
     if b"<" not in text and b">" not in text:
         username = text
         email = text
+    elif b">" not in text:
+        return text + b">"
     else:
         if text.rindex(b">") < text.rindex(b"<"):
             raise ValueError(text)
@@ -131,6 +126,16 @@ def fix_person_identifier(text):
     return b"%s <%s>" % (username, email)
 
 
+def decode_git_path(path):
+    """Take a git path and decode it."""
+    return path.decode('utf-8', 'surrogateescape')
+
+
+def encode_git_path(path):
+    """Take a regular path and encode it for git."""
+    return path.encode('utf-8', 'surrogateescape')
+
+
 def warn_escaped(commit, num_escaped):
     trace.warning("Escaped %d XML-invalid characters in %s. Will be unable "
                   "to regenerate the SHA map.", num_escaped, commit)
@@ -145,7 +150,7 @@ class BzrGitMapping(foreign.VcsMapping):
     """Class that maps between Git and Bazaar semantics."""
     experimental = False
 
-    BZR_DUMMY_FILE = None
+    BZR_DUMMY_FILE = None  # type: Optional[str]
 
     def is_special_file(self, filename):
         return (filename in (self.BZR_DUMMY_FILE, ))
@@ -175,8 +180,8 @@ class BzrGitMapping(foreign.VcsMapping):
     def generate_file_id(self, path):
         # Git paths are just bytestrings
         # We must just hope they are valid UTF-8..
-        if isinstance(path, text_type):
-            path = path.encode("utf-8")
+        if isinstance(path, str):
+            path = encode_git_path(path)
         if path == b"":
             return ROOT_ID
         return FILE_ID_PREFIX + escape_file_id(path)
@@ -186,15 +191,7 @@ class BzrGitMapping(foreign.VcsMapping):
             return u""
         if not file_id.startswith(FILE_ID_PREFIX):
             raise ValueError
-        return unescape_file_id(file_id[len(FILE_ID_PREFIX):]).decode('utf-8')
-
-    def revid_as_refname(self, revid):
-        if not isinstance(revid, bytes):
-            raise TypeError(revid)
-        if PY3:
-            revid = revid.decode('utf-8')
-        quoted_revid = urlutils.quote(revid)
-        return b"refs/bzr/" + quoted_revid.encode('utf-8')
+        return decode_git_path(unescape_file_id(file_id[len(FILE_ID_PREFIX):]))
 
     def import_unusual_file_modes(self, rev, unusual_file_modes):
         if unusual_file_modes:
@@ -252,11 +249,11 @@ class BzrGitMapping(foreign.VcsMapping):
         (message, renames, branch, extra) = extract_hg_metadata(message)
         if branch is not None:
             rev.properties[u'hg:extra:branch'] = branch
-        for name, value in viewitems(extra):
+        for name, value in extra.items():
             rev.properties[u'hg:extra:' + name] = base64.b64encode(value)
         if renames:
             rev.properties[u'hg:renames'] = base64.b64encode(bencode.bencode(
-                [(new, old) for (old, new) in viewitems(renames)]))
+                [(new, old) for (old, new) in renames.items()]))
         return message
 
     def _extract_bzr_metadata(self, rev, message):
@@ -332,9 +329,7 @@ class BzrGitMapping(foreign.VcsMapping):
             commit.author_timezone = commit.commit_timezone
         if u'git-gpg-signature' in rev.properties:
             commit.gpgsig = rev.properties[u'git-gpg-signature'].encode(
-                'utf-8')
-        if u'git-gpg-signature-b64' in rev.properties:
-            commit.gpgsig = base64.b64decode(rev.properties[u'git-gpg-signature-b64'])
+                'utf-8', 'surrogateescape')
         commit.message = self._encode_commit_message(rev, rev.message,
                                                      encoding)
         if not isinstance(commit.message, bytes):
@@ -347,10 +342,9 @@ class BzrGitMapping(foreign.VcsMapping):
             mapping_properties = set(
                 [u'author', u'author-timezone', u'author-timezone-neg-utc',
                  u'commit-timezone-neg-utc', u'git-implicit-encoding',
-                 u'git-gpg-signature', u'git-gpg-signature-b64',
-                 u'git-explicit-encoding',
+                 u'git-gpg-signature', u'git-explicit-encoding',
                  u'author-timestamp', u'file-modes'])
-            for k, v in viewitems(rev.properties):
+            for k, v in rev.properties.items():
                 if k not in mapping_properties:
                     metadata.properties[k] = v
         if not lossy and metadata:
@@ -407,9 +401,11 @@ class BzrGitMapping(foreign.VcsMapping):
                 rev.properties[u'author'] = commit.author.decode(encoding)
             rev.message, rev.git_metadata = self._decode_commit_message(
                 rev, commit.message, encoding)
+
         if commit.encoding is not None:
             rev.properties[u'git-explicit-encoding'] = commit.encoding.decode(
                 'ascii')
+        if commit.encoding is not None and commit.encoding != b'false':
             decode_using_encoding(rev, commit, commit.encoding.decode('ascii'))
         else:
             for encoding in ('utf-8', 'latin1'):
@@ -430,12 +426,8 @@ class BzrGitMapping(foreign.VcsMapping):
         if commit._commit_timezone_neg_utc:
             rev.properties[u'commit-timezone-neg-utc'] = ""
         if commit.gpgsig:
-            try:
-                rev.properties[u'git-gpg-signature'] = commit.gpgsig.decode(
-                    'utf-8')
-            except UnicodeDecodeError:
-                rev.properties[u'git-gpg-signature-b64'] = base64.b64encode(
-                    commit.gpgsig)
+            rev.properties[u'git-gpg-signature'] = commit.gpgsig.decode(
+                'utf-8', 'surrogateescape')
         if commit.mergetag:
             for i, tag in enumerate(commit.mergetag):
                 rev.properties[u'git-mergetag-%d' % i] = tag.as_raw_string()
@@ -581,8 +573,8 @@ default_mapping = mapping_registry.get_default()()
 def symlink_to_blob(symlink_target):
     from dulwich.objects import Blob
     blob = Blob()
-    if isinstance(symlink_target, text_type):
-        symlink_target = symlink_target.encode('utf-8')
+    if isinstance(symlink_target, str):
+        symlink_target = encode_git_path(symlink_target)
     blob.data = symlink_target
     return blob
 
