@@ -16,8 +16,6 @@
 
 """Support for GitHub."""
 
-from __future__ import absolute_import
-
 import json
 import os
 
@@ -44,10 +42,13 @@ from ... import (
     version_string as breezy_version,
     )
 from ...config import AuthenticationConfig, GlobalStack
-from ...errors import InvalidHttpResponse, PermissionDenied
+from ...errors import (
+    InvalidHttpResponse,
+    PermissionDenied,
+    UnexpectedHttpStatus,
+    )
 from ...git.urls import git_url_to_bzr_url
 from ...i18n import gettext
-from ...sixish import PY3
 from ...trace import note
 from ...transport import get_transport
 from ...transport.http import default_user_agent
@@ -123,6 +124,8 @@ class GitHubMergeProposal(MergeProposal):
     def __repr__(self):
         return "<%s at %r>" % (type(self).__name__, self.url)
 
+    name = 'GitHub'
+
     @property
     def url(self):
         return self._pr['html_url']
@@ -135,6 +138,12 @@ class GitHubMergeProposal(MergeProposal):
     def get_source_branch_url(self):
         return self._branch_from_part(self._pr['head'])
 
+    def get_source_revision(self):
+        """Return the latest revision for the source branch."""
+        from breezy.git.mapping import default_mapping
+        return default_mapping.revision_id_foreign_to_bzr(
+            self._pr['head']['sha'].encode('ascii'))
+
     def get_target_branch_url(self):
         return self._branch_from_part(self._pr['base'])
 
@@ -159,7 +168,7 @@ class GitHubMergeProposal(MergeProposal):
         if response.status == 422:
             raise ValidationFailed(json.loads(response.text))
         if response.status != 200:
-            raise InvalidHttpResponse(self._pr['url'], response.text)
+            raise UnexpectedHttpStatus(self._pr['url'], response.status)
         self._pr = json.loads(response.text)
 
     def set_description(self, description):
@@ -188,7 +197,15 @@ class GitHubMergeProposal(MergeProposal):
 
     def merge(self, commit_message=None):
         # https://developer.github.com/v3/pulls/#merge-a-pull-request-merge-button
-        self._pr.merge(commit_message=commit_message)
+        data = {}
+        if commit_message:
+            data['commit_message'] = commit_messae
+        response = self._gh._api_request(
+            'PUT', self._pr['url'] + "/merge", body=json.dumps(data).encode('utf-8'))
+        if response.status == 422:
+            raise ValidationFailed(json.loads(response.text))
+        if response.status != 200:
+            raise UnexpectedHttpStatus(self._pr['url'], response.status)
 
     def get_merged_by(self):
         merged_by = self._pr.get('merged_by')
@@ -203,6 +220,17 @@ class GitHubMergeProposal(MergeProposal):
         import iso8601
         return iso8601.parse_date(merged_at)
 
+    def post_comment(self, body):
+        data = {'body': body}
+        response = self._gh._api_request(
+            'POST', self._pr['comments_url'], body=json.dumps(data).encode('utf-8'))
+        if response.status == 422:
+            raise ValidationFailed(json.loads(response.text))
+        if response.status != 201:
+            raise UnexpectedHttpStatus(
+                self._pr['comments_url'], response.status)
+        json.loads(response.text)
+
 
 def parse_github_url(url):
     (scheme, user, password, host, port, path) = urlutils.parse_url(
@@ -222,8 +250,6 @@ def parse_github_branch_url(branch):
 
 
 def github_url_to_bzr_url(url, branch_name):
-    if not PY3:
-        branch_name = branch_name.encode('utf-8')
     return git_url_to_bzr_url(url, branch_name)
 
 
@@ -249,11 +275,17 @@ class GitHub(Hoster):
             'Accept': 'application/vnd.github.v3+json'}
         if self._token:
             headers['Authorization'] = 'token %s' % self._token
-        response = self.transport.request(
-            method, urlutils.join(self.transport.base, path),
-            headers=headers, body=body, retries=3)
+        try:
+            response = self.transport.request(
+                method, urlutils.join(self.transport.base, path),
+                headers=headers, body=body, retries=3)
+        except UnexpectedHttpStatus as e:
+            if e.code == 401:
+                raise GitHubLoginRequired(self.base_url)
+            else:
+                raise
         if response.status == 401:
-            raise GitHubLoginRequired(self)
+            raise GitHubLoginRequired(self.base_url)
         return response
 
     def _get_repo(self, owner, repo):
@@ -263,7 +295,7 @@ class GitHub(Hoster):
             raise NoSuchProject(path)
         if response.status == 200:
             return json.loads(response.text)
-        raise InvalidHttpResponse(path, response.text)
+        raise UnexpectedHttpStatus(path, response.status)
 
     def _get_repo_pulls(self, path, head=None, state=None):
         path = path + '?'
@@ -279,7 +311,7 @@ class GitHub(Hoster):
             raise NoSuchProject(path)
         if response.status == 200:
             return json.loads(response.text)
-        raise InvalidHttpResponse(path, response.text)
+        raise UnexpectedHttpStatus(path, response.status)
 
     def _create_pull(self, path, title, head, base, body=None, labels=None,
                      assignee=None, draft=False, maintainer_can_modify=False):
@@ -301,15 +333,17 @@ class GitHub(Hoster):
             'POST', path, body=json.dumps(data).encode('utf-8'))
         if response.status == 403:
             raise PermissionDenied(path, response.text)
+        if response.status == 422:
+            raise ValidationFailed(json.loads(response.text))
         if response.status != 201:
-            raise InvalidHttpResponse(path, 'req is invalid %d %r: %r' % (response.status, data, response.text))
+            raise UnexpectedHttpStatus(path, response.status)
         return json.loads(response.text)
 
     def _get_user_by_email(self, email):
         path = 'search/users?q=%s+in:email' % email
         response = self._api_request('GET', path)
         if response.status != 200:
-            raise InvalidHttpResponse(path, response.text)
+            raise UnexpectedHttpStatus(path, response.status)
         ret = json.loads(response.text)
         if ret['total_count'] == 0:
             raise KeyError('no user with email %s' % email)
@@ -324,14 +358,14 @@ class GitHub(Hoster):
             path = 'user'
         response = self._api_request('GET', path)
         if response.status != 200:
-            raise InvalidHttpResponse(path, response.text)
+            raise UnexpectedHttpStatus(path, response.status)
         return json.loads(response.text)
 
     def _get_organization(self, name):
         path = 'orgs/%s' % name
         response = self._api_request('GET', path)
         if response.status != 200:
-            raise InvalidHttpResponse(path, response.text)
+            raise UnexpectedHttpStatus(path, response.status)
         return json.loads(response.text)
 
     def _list_paged(self, path, parameters=None, per_page=None):
@@ -350,7 +384,7 @@ class GitHub(Hoster):
                 ';'.join(['%s=%s' % (k, urlutils.quote(v))
                           for (k, v) in parameters.items()]))
             if response.status != 200:
-                raise InvalidHttpResponse(path, response.text)
+                raise UnexpectedHttpStatus(path, response.status)
             data = json.loads(response.text)
             for entry in data['items']:
                 i += 1
@@ -364,11 +398,11 @@ class GitHub(Hoster):
         return self._list_paged(path, {'q': query}, per_page=DEFAULT_PER_PAGE)
 
     def _create_fork(self, path, owner=None):
-        if owner and owner != self._current_user['login']:
+        if owner and owner != self.current_user['login']:
             path += '?organization=%s' % owner
         response = self._api_request('POST', path)
         if response.status != 202:
-            raise InvalidHttpResponse(path, 'status: %d, %r' % (response.status, response.text))
+            raise UnexpectedHttpStatus(path, response.status)
         return json.loads(response.text)
 
     @property
@@ -378,7 +412,13 @@ class GitHub(Hoster):
     def __init__(self, transport):
         self._token = retrieve_github_token('https', GITHUB_HOST)
         self.transport = transport
-        self._current_user = self._get_user()
+        self._current_user = None
+
+    @property
+    def current_user(self):
+        if self._current_user is None:
+            self._current_user = self._get_user()
+        return self._current_user
 
     def publish_derived(self, local_branch, base_branch, name, project=None,
                         owner=None, revision_id=None, overwrite=False,
@@ -386,7 +426,7 @@ class GitHub(Hoster):
         base_owner, base_project, base_branch_name = parse_github_branch_url(base_branch)
         base_repo = self._get_repo(base_owner, base_project)
         if owner is None:
-            owner = self._current_user['login']
+            owner = self.current_user['login']
         if project is None:
             project = base_repo['name']
         try:
@@ -418,19 +458,33 @@ class GitHub(Hoster):
         repo = self._get_repo(owner, project)
         return github_url_to_bzr_url(repo['ssh_url'], branch_name)
 
-    def get_derived_branch(self, base_branch, name, project=None, owner=None):
+    def get_derived_branch(self, base_branch, name, project=None, owner=None, preferred_schemes=None):
         base_owner, base_project, base_branch_name = parse_github_branch_url(base_branch)
         base_repo = self._get_repo(base_owner, base_project)
         if owner is None:
-            owner = self._current_user['login']
+            owner = self.current_user['login']
         if project is None:
             project = base_repo['name']
         try:
             remote_repo = self._get_repo(owner, project)
-            full_url = github_url_to_bzr_url(remote_repo['ssh_url'], name)
-            return _mod_branch.Branch.open(full_url)
         except NoSuchProject:
             raise errors.NotBranchError('%s/%s/%s' % (WEB_GITHUB_URL, owner, project))
+        if preferred_schemes is None:
+            preferred_schemes = ['git+ssh']
+        for scheme in preferred_schemes:
+            if scheme == 'git+ssh':
+                github_url = remote_repo['ssh_url']
+                break
+            if scheme == 'https':
+                github_url = remote_repo['clone_url']
+                break
+            if scheme == 'git':
+                github_url = remote_repo['git_url']
+                break
+        else:
+            raise AssertionError
+        full_url = github_url_to_bzr_url(github_url, name)
+        return _mod_branch.Branch.open(full_url)
 
     def get_proposer(self, source_branch, target_branch):
         return GitHubMergeProposalBuilder(self, source_branch, target_branch)
@@ -486,7 +540,7 @@ class GitHub(Hoster):
     def iter_instances(cls):
         yield cls(get_transport(API_GITHUB_URL))
 
-    def iter_my_proposals(self, status='open'):
+    def iter_my_proposals(self, status='open', author=None):
         query = ['is:pr']
         if status == 'open':
             query.append('is:open')
@@ -497,21 +551,27 @@ class GitHub(Hoster):
             query.append('is:closed')
         elif status == 'merged':
             query.append('is:merged')
-        query.append('author:%s' % self._current_user['login'])
+        if author is None:
+            author = self.current_user['login']
+        query.append('author:%s' % author)
         for issue in self._search_issues(query=' '.join(query)):
             url = issue['pull_request']['url']
             response = self._api_request('GET', url)
             if response.status != 200:
-                raise InvalidHttpResponse(url, response.text)
+                raise UnexpectedHttpStatus(url, response.status)
             yield GitHubMergeProposal(self, json.loads(response.text))
 
     def get_proposal_by_url(self, url):
         raise UnsupportedHoster(url)
 
-    def iter_my_forks(self):
-        response = self._api_request('GET', '/user/repos')
+    def iter_my_forks(self, owner=None):
+        if owner:
+            path = '/users/%s/repos' % owner
+        else:
+            path = '/user/repos'
+        response = self._api_request('GET', path)
         if response.status != 200:
-            raise InvalidHttpResponse(url, response.text)
+            raise UnexpectedHttpStatus(self.transport.user_url, response.status)
         for project in json.loads(response.text):
             if not project['fork']:
                 continue
@@ -526,7 +586,15 @@ class GitHub(Hoster):
             return
         if response.status == 200:
             return json.loads(response.text)
-        raise InvalidHttpResponse(path, response.text)
+        raise UnexpectedHttpStatus(path, response.status)
+
+    def get_current_user(self):
+        if self._token is not None:
+            return self.current_user['login']
+        return None
+
+    def get_user_url(self, username):
+        return urlutils.join(self.base_url, username)
 
 
 class GitHubMergeProposalBuilder(MergeProposalBuilder):
@@ -593,5 +661,7 @@ class GitHubMergeProposalBuilder(MergeProposalBuilder):
                 maintainer_can_modify=allow_collaboration,
                 )
         except ValidationFailed:
+            # TODO(jelmer): Check the actual error message rather than assuming
+            # a merge proposal exists?
             raise MergeProposalExists(self.source_branch.user_url)
         return GitHubMergeProposal(self.gh, pull_request)
