@@ -14,8 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from __future__ import absolute_import
-
 import errno
 import re
 
@@ -68,10 +66,10 @@ class ShelfCreator(object):
         :param file_list: The files to make more similar to the target.
         """
         self.work_tree = work_tree
-        self.work_transform = work_tree.get_transform()
+        self.work_transform = work_tree.transform()
         try:
             self.target_tree = target_tree
-            self.shelf_transform = transform.TransformPreview(self.target_tree)
+            self.shelf_transform = self.target_tree.preview_transform()
             try:
                 self.renames = {}
                 self.creation = {}
@@ -268,7 +266,7 @@ class ShelfCreator(object):
                         to_transform, s_trans_id, tree,
                         tree.id2path(file_id))
         if version:
-            to_transform.version_file(file_id, s_trans_id)
+            to_transform.version_file(s_trans_id, file_id=file_id)
 
     def _inverse_lines(self, new_lines, file_id):
         """Produce a version with only those changes removed from new_lines."""
@@ -365,7 +363,7 @@ class Unshelver(object):
             base_tree = tree.revision_tree(base_revision_id)
         except errors.NoSuchRevisionInTree:
             base_tree = tree.branch.repository.revision_tree(base_revision_id)
-        tt = transform.TransformPreview(base_tree)
+        tt = base_tree.preview_transform()
         tt.deserialize(records)
         return klass(tree, base_tree, tt, metadata.get(b'message'))
 
