@@ -31,13 +31,12 @@ from ... import (
     msgeditor,
     )
 from ...controldir import ControlDir
-from ...sixish import PY3
 from .. import (
     test_foreign,
     features,
     )
 from .. import TestCaseWithTransport
-from ..matchers import ContainsNoVfsCalls
+from ..test_bedding import override_whoami
 
 
 class TestCommit(TestCaseWithTransport):
@@ -146,7 +145,7 @@ brz: ERROR: No changes to commit.\
         out, err = self.run_bzr(['commit', '-m', file_name])
         reflags = re.MULTILINE | re.DOTALL | re.UNICODE
         te = osutils.get_terminal_encoding()
-        self.assertContainsRe(err if PY3 else err.decode(te),
+        self.assertContainsRe(err,
                               u'The commit message is a file name:',
                               flags=reflags)
 
@@ -165,7 +164,7 @@ brz: ERROR: No changes to commit.\
             out, err = self.run_bzr(['commit', '-m', file_name])
             reflags = re.MULTILINE | re.DOTALL | re.UNICODE
             te = osutils.get_terminal_encoding()
-            self.assertContainsRe(err if PY3 else err.decode(te, 'replace'),
+            self.assertContainsRe(err,
                                   u'The commit message is a file name:',
                                   flags=reflags)
         finally:
@@ -187,11 +186,7 @@ brz: ERROR: No changes to commit.\
         tree.add(["f"])
         out, err = self.run_bzr_raw(["commit", "-m", "Wrong filename", u"\xa7"],
                                     encoding="iso-8859-5", retcode=3)
-        if not PY3:
-            self.expectFailure("Error messages are always written as UTF-8",
-                               self.assertNotContainsString, err, b"\xc2\xa7")
-        else:
-            self.assertNotContainsString(err, b"\xc2\xa7")
+        self.assertNotContainsString(err, b"\xc2\xa7")
         self.assertContainsRe(err, b"(?m)not versioned: \"\xfd\"$")
 
     def test_warn_about_forgotten_commit_message(self):
@@ -315,9 +310,8 @@ brz: ERROR: No changes to commit.\
         this_tree.commit('create_files')
         other_dir = this_tree.controldir.sprout('other')
         other_tree = other_dir.open_workingtree()
-        other_tree.lock_write()
-        # perform the needed actions on the files and dirs.
-        try:
+        with other_tree.lock_write():
+            # perform the needed actions on the files and dirs.
             other_tree.rename_one('dirtorename', 'renameddir')
             other_tree.rename_one('dirtoreparent', 'renameddir/reparenteddir')
             other_tree.rename_one('filetorename', 'renamedfile')
@@ -331,8 +325,6 @@ brz: ERROR: No changes to commit.\
             other_tree.add('newfile')
             other_tree.add('newdir/')
             other_tree.commit('modify all sample files and dirs.')
-        finally:
-            other_tree.unlock()
         this_tree.merge_from_branch(other_tree.branch)
         out, err = self.run_bzr('commit -m added', working_dir='this')
         self.assertEqual('', out)
@@ -868,11 +860,7 @@ altered in u2
         with open('foo/foo.txt', 'w') as f:
             f.write('hello')
         self.run_bzr(['add'], working_dir='foo')
-        self.overrideEnv('EMAIL', None)
-        self.overrideEnv('BRZ_EMAIL', None)
-        # Also, make sure that it's not inferred from mailname.
-        self.overrideAttr(config, '_auto_user_id',
-                          lambda: (None, None))
+        override_whoami(self)
         self.run_bzr_error(
             ['Unable to determine your name'],
             ['commit', '-m', 'initial'], working_dir='foo')
@@ -906,26 +894,3 @@ altered in u2
 
         self.run_bzr('ci -m "non-ascii mv"')
 
-
-class TestSmartServerCommit(TestCaseWithTransport):
-
-    def test_commit_to_lightweight(self):
-        self.setup_smart_server_with_call_log()
-        t = self.make_branch_and_tree('from')
-        for count in range(9):
-            t.commit(message='commit %d' % count)
-        out, err = self.run_bzr(['checkout', '--lightweight', self.get_url('from'),
-                                 'target'])
-        self.reset_smart_call_log()
-        self.build_tree(['target/afile'])
-        self.run_bzr(['add', 'target/afile'])
-        out, err = self.run_bzr(['commit', '-m', 'do something', 'target'])
-        # This figure represent the amount of work to perform this use case. It
-        # is entirely ok to reduce this number if a test fails due to rpc_count
-        # being too low. If rpc_count increases, more network roundtrips have
-        # become necessary for this use case. Please do not adjust this number
-        # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(211, self.hpss_calls)
-        self.assertLength(2, self.hpss_connections)
-        self.expectFailure("commit still uses VFS calls",
-                           self.assertThat, self.hpss_calls, ContainsNoVfsCalls)
