@@ -19,10 +19,7 @@
 Only one dependency: ctypes should be installed.
 """
 
-from __future__ import absolute_import
-
 import glob
-import operator
 import os
 import struct
 import sys
@@ -32,37 +29,6 @@ from breezy import (
     )
 from breezy.i18n import gettext
 
-# We can cope without it; use a separate variable to help pyflakes
-try:
-    import ctypes
-    has_ctypes = True
-except ImportError:
-    has_ctypes = False
-else:
-    create_buffer = ctypes.create_unicode_buffer
-    extract_buffer = operator.attrgetter("value")
-    suffix = 'W'
-try:
-    import pywintypes
-    has_pywintypes = True
-except ImportError:
-    has_pywintypes = has_win32file = has_win32api = False
-else:
-    try:
-        import win32file
-        has_win32file = True
-    except ImportError:
-        has_win32file = False
-    try:
-        import win32api
-        has_win32api = True
-    except ImportError:
-        has_win32api = False
-
-# pulling in win32com.shell is a bit of overhead, and normally we don't need
-# it as ctypes is preferred and common.  lazy_imports and "optional"
-# modules don't work well, so we do our own lazy thing...
-has_win32com_shell = None  # Set to True or False once we know for sure...
 
 # Special Win32 API constants
 # Handles of std streams
@@ -88,51 +54,41 @@ REG_EXPAND_SZ = 2
 
 def debug_memory_win32api(message='', short=True):
     """Use trace.note() to dump the running memory info."""
+    import ctypes
     from breezy import trace
-    if has_ctypes:
-        class PROCESS_MEMORY_COUNTERS_EX(ctypes.Structure):
-            """Used by GetProcessMemoryInfo"""
-            _fields_ = [('cb', ctypes.c_ulong),
-                        ('PageFaultCount', ctypes.c_ulong),
-                        ('PeakWorkingSetSize', ctypes.c_size_t),
-                        ('WorkingSetSize', ctypes.c_size_t),
-                        ('QuotaPeakPagedPoolUsage', ctypes.c_size_t),
-                        ('QuotaPagedPoolUsage', ctypes.c_size_t),
-                        ('QuotaPeakNonPagedPoolUsage', ctypes.c_size_t),
-                        ('QuotaNonPagedPoolUsage', ctypes.c_size_t),
-                        ('PagefileUsage', ctypes.c_size_t),
-                        ('PeakPagefileUsage', ctypes.c_size_t),
-                        ('PrivateUsage', ctypes.c_size_t),
-                        ]
-        cur_process = ctypes.windll.kernel32.GetCurrentProcess()
-        mem_struct = PROCESS_MEMORY_COUNTERS_EX()
-        ret = ctypes.windll.psapi.GetProcessMemoryInfo(
-            cur_process, ctypes.byref(mem_struct), ctypes.sizeof(mem_struct))
-        if not ret:
-            trace.note(gettext('Failed to GetProcessMemoryInfo()'))
-            return
-        info = {'PageFaultCount': mem_struct.PageFaultCount,
-                'PeakWorkingSetSize': mem_struct.PeakWorkingSetSize,
-                'WorkingSetSize': mem_struct.WorkingSetSize,
-                'QuotaPeakPagedPoolUsage': mem_struct.QuotaPeakPagedPoolUsage,
-                'QuotaPagedPoolUsage': mem_struct.QuotaPagedPoolUsage,
-                'QuotaPeakNonPagedPoolUsage':
-                    mem_struct.QuotaPeakNonPagedPoolUsage,
-                'QuotaNonPagedPoolUsage': mem_struct.QuotaNonPagedPoolUsage,
-                'PagefileUsage': mem_struct.PagefileUsage,
-                'PeakPagefileUsage': mem_struct.PeakPagefileUsage,
-                'PrivateUsage': mem_struct.PrivateUsage,
-                }
-    elif has_win32api:
-        import win32process
-        # win32process does not return PrivateUsage, because it doesn't use
-        # PROCESS_MEMORY_COUNTERS_EX (it uses the one without _EX).
-        proc = win32process.GetCurrentProcess()
-        info = win32process.GetProcessMemoryInfo(proc)
-    else:
-        trace.note(gettext('Cannot debug memory on win32 without ctypes'
-                           ' or win32process'))
+    class PROCESS_MEMORY_COUNTERS_EX(ctypes.Structure):
+        """Used by GetProcessMemoryInfo"""
+        _fields_ = [('cb', ctypes.c_ulong),
+                    ('PageFaultCount', ctypes.c_ulong),
+                    ('PeakWorkingSetSize', ctypes.c_size_t),
+                    ('WorkingSetSize', ctypes.c_size_t),
+                    ('QuotaPeakPagedPoolUsage', ctypes.c_size_t),
+                    ('QuotaPagedPoolUsage', ctypes.c_size_t),
+                    ('QuotaPeakNonPagedPoolUsage', ctypes.c_size_t),
+                    ('QuotaNonPagedPoolUsage', ctypes.c_size_t),
+                    ('PagefileUsage', ctypes.c_size_t),
+                    ('PeakPagefileUsage', ctypes.c_size_t),
+                    ('PrivateUsage', ctypes.c_size_t),
+                    ]
+    cur_process = ctypes.windll.kernel32.GetCurrentProcess()
+    mem_struct = PROCESS_MEMORY_COUNTERS_EX()
+    ret = ctypes.windll.psapi.GetProcessMemoryInfo(
+        cur_process, ctypes.byref(mem_struct), ctypes.sizeof(mem_struct))
+    if not ret:
+        trace.note(gettext('Failed to GetProcessMemoryInfo()'))
         return
+    info = {'PageFaultCount': mem_struct.PageFaultCount,
+            'PeakWorkingSetSize': mem_struct.PeakWorkingSetSize,
+            'WorkingSetSize': mem_struct.WorkingSetSize,
+            'QuotaPeakPagedPoolUsage': mem_struct.QuotaPeakPagedPoolUsage,
+            'QuotaPagedPoolUsage': mem_struct.QuotaPagedPoolUsage,
+            'QuotaPeakNonPagedPoolUsage':
+                mem_struct.QuotaPeakNonPagedPoolUsage,
+            'QuotaNonPagedPoolUsage': mem_struct.QuotaNonPagedPoolUsage,
+            'PagefileUsage': mem_struct.PagefileUsage,
+            'PeakPagefileUsage': mem_struct.PeakPagefileUsage,
+            'PrivateUsage': mem_struct.PrivateUsage,
+            }
     if short:
         # using base-2 units (see HACKING.txt).
         trace.note(gettext('WorkingSize {0:>7}KiB'
@@ -163,10 +119,7 @@ def get_console_size(defaultx=80, defaulty=25):
     console window and return tuple (sizex, sizey) if success,
     or default size (defaultx, defaulty) otherwise.
     """
-    if not has_ctypes:
-        # no ctypes is found
-        return (defaultx, defaulty)
-
+    import ctypes
     # To avoid problem with redirecting output via pipe
     # we need to use stderr instead of stdout
     h = ctypes.windll.kernel32.GetStdHandle(WIN32_STDERR_HANDLE)
@@ -189,34 +142,16 @@ def _get_sh_special_folder_path(csidl):
 
     Result is always unicode (or None).
     """
-    if has_ctypes:
-        try:
-            SHGetSpecialFolderPath = \
-                ctypes.windll.shell32.SHGetSpecialFolderPathW
-        except AttributeError:
-            pass
-        else:
-            buf = ctypes.create_unicode_buffer(MAX_PATH)
-            if SHGetSpecialFolderPath(None, buf, csidl, 0):
-                return buf.value
-
-    global has_win32com_shell
-    if has_win32com_shell is None:
-        try:
-            from win32com.shell import shell
-            has_win32com_shell = True
-        except ImportError:
-            has_win32com_shell = False
-    if has_win32com_shell:
-        # still need to bind the name locally, but this is fast.
-        from win32com.shell import shell
-        try:
-            return shell.SHGetSpecialFolderPath(0, csidl, 0)
-        except shell.error:
-            # possibly E_NOTIMPL meaning we can't load the function pointer,
-            # or E_FAIL meaning the function failed - regardless, just ignore it
-            pass
-    return None
+    import ctypes
+    try:
+        SHGetSpecialFolderPath = \
+            ctypes.windll.shell32.SHGetSpecialFolderPathW
+    except AttributeError:
+        pass
+    else:
+        buf = ctypes.create_unicode_buffer(MAX_PATH)
+        if SHGetSpecialFolderPath(None, buf, csidl, 0):
+            return buf.value
 
 
 def get_appdata_location():
@@ -232,7 +167,7 @@ def get_appdata_location():
     if appdata:
         return appdata
     # Use APPDATA if defined, will return None if not
-    return get_environ_unicode('APPDATA')
+    return os.environ.get('APPDATA')
 
 
 def get_local_appdata_location():
@@ -249,7 +184,7 @@ def get_local_appdata_location():
     if local:
         return local
     # Vista supplies LOCALAPPDATA, but XP and earlier do not.
-    local = get_environ_unicode('LOCALAPPDATA')
+    local = os.environ.get('LOCALAPPDATA')
     if local:
         return local
     return get_appdata_location()
@@ -264,14 +199,14 @@ def get_home_location():
     home = _get_sh_special_folder_path(CSIDL_PERSONAL)
     if home:
         return home
-    home = get_environ_unicode('HOME')
+    home = os.environ.get('HOME')
     if home is not None:
         return home
-    homepath = get_environ_unicode('HOMEPATH')
+    homepath = os.environ.get('HOMEPATH')
     if homepath is not None:
-        return os.path.join(get_environ_unicode('HOMEDIR', ''), home)
+        return os.path.join(os.environ.get('HOMEDIR', ''), home)
     # at least return windows root directory
-    windir = get_environ_unicode('WINDIR')
+    windir = os.environ.get('WINDIR')
     if windir:
         return os.path.splitdrive(windir)[0] + '/'
     # otherwise C:\ is good enough for 98% users
@@ -282,19 +217,18 @@ def get_user_name():
     """Return user name as login name.
     If name cannot be obtained return None.
     """
-    if has_ctypes:
-        try:
-            advapi32 = ctypes.windll.advapi32
-            GetUserName = getattr(advapi32, 'GetUserName' + suffix)
-        except AttributeError:
-            pass
-        else:
-            buf = create_buffer(UNLEN + 1)
-            n = ctypes.c_int(UNLEN + 1)
-            if GetUserName(buf, ctypes.byref(n)):
-                return extract_buffer(buf)
+    try:
+        advapi32 = ctypes.windll.advapi32
+        GetUserName = getattr(advapi32, 'GetUserNameW')
+    except AttributeError:
+        pass
+    else:
+        buf = ctypes.create_unicode_buffer(UNLEN + 1)
+        n = ctypes.c_int(UNLEN + 1)
+        if GetUserName(buf, ctypes.byref(n)):
+            return buf.value
     # otherwise try env variables
-    return get_environ_unicode('USERNAME')
+    return os.environ.get('USERNAME')
 
 
 # 1 == ComputerNameDnsHostname, which returns "The DNS host name of the local
@@ -308,39 +242,18 @@ def get_host_name():
 
     :return: A unicode string representing the host name.
     """
-    if has_win32api:
-        try:
-            return win32api.GetComputerNameEx(_WIN32_ComputerNameDnsHostname)
-        except (NotImplementedError, win32api.error):
-            # NotImplemented will happen on win9x...
-            pass
-    if has_ctypes:
-        try:
-            kernel32 = ctypes.windll.kernel32
-        except AttributeError:
-            pass  # Missing the module we need
-        else:
-            buf = create_buffer(MAX_COMPUTERNAME_LENGTH + 1)
-            n = ctypes.c_int(MAX_COMPUTERNAME_LENGTH + 1)
+    import ctypes
+    buf = ctypes.create_unicode_buffer(MAX_COMPUTERNAME_LENGTH + 1)
+    n = ctypes.c_int(MAX_COMPUTERNAME_LENGTH + 1)
 
-            # Try GetComputerNameEx which gives a proper Unicode hostname
-            GetComputerNameEx = getattr(kernel32, 'GetComputerNameEx' + suffix,
-                                        None)
-            if (GetComputerNameEx is not None
-                and GetComputerNameEx(_WIN32_ComputerNameDnsHostname,
-                                      buf, ctypes.byref(n))):
-                return extract_buffer(buf)
-
-            # Try GetComputerName in case GetComputerNameEx wasn't found
-            # It returns the NETBIOS name, which isn't as good, but still ok.
-            # The first GetComputerNameEx might have changed 'n', so reset it
-            n = ctypes.c_int(MAX_COMPUTERNAME_LENGTH + 1)
-            GetComputerName = getattr(kernel32, 'GetComputerName' + suffix,
-                                      None)
-            if (GetComputerName is not None
-                    and GetComputerName(buf, ctypes.byref(n))):
-                return extract_buffer(buf)
-    return get_environ_unicode('COMPUTERNAME')
+    # Try GetComputerNameEx which gives a proper Unicode hostname
+    GetComputerNameEx = getattr(
+        ctypes.windll.kernel32, 'GetComputerNameExW', None)
+    if (GetComputerNameEx is not None
+        and GetComputerNameEx(_WIN32_ComputerNameDnsHostname,
+                              buf, ctypes.byref(n))):
+        return buf.value
+    return os.environ.get('COMPUTERNAME')
 
 
 def _ensure_with_dir(path):
@@ -430,23 +343,22 @@ def get_app_path(appname):
 
     if type_id == REG_SZ:
         return path
-    if type_id == REG_EXPAND_SZ and has_win32api:
-        fullpath = win32api.ExpandEnvironmentStrings(path)
-        if len(fullpath) > 1 and fullpath[0] == '"' and fullpath[-1] == '"':
-            fullpath = fullpath[1:-1]   # remove quotes around value
-        return fullpath
     return appname
 
 
 def set_file_attr_hidden(path):
     """Set file attributes to hidden if possible"""
-    if has_win32file:
-        SetFileAttributes = win32file.SetFileAttributesW
-        try:
-            SetFileAttributes(path, win32file.FILE_ATTRIBUTE_HIDDEN)
-        except pywintypes.error as e:
-            from . import trace
-            trace.mutter('Unable to set hidden attribute on %r: %s', path, e)
+    from ctypes.wintypes import BOOL, DWORD, LPWSTR
+    import ctypes
+    # <https://docs.microsoft.com/windows/desktop/api/fileapi/nf-fileapi-setfileattributesw>
+    SetFileAttributes = ctypes.windll.kernel32.SetFileAttributesW
+    SetFileAttributes.argtypes = LPWSTR, DWORD
+    SetFileAttributes.restype = BOOL
+    FILE_ATTRIBUTE_HIDDEN = 2
+    if not SetFileAttributes(path, FILE_ATTRIBUTE_HIDDEN):
+        e = ctypes.WinError()
+        from . import trace
+        trace.mutter('Unable to set hidden attribute on %r: %s', path, e)
 
 
 def _command_line_to_argv(command_line, argv, single_quotes_allowed=False):
@@ -491,92 +403,20 @@ def _command_line_to_argv(command_line, argv, single_quotes_allowed=False):
     return args
 
 
-if has_ctypes:
-    def get_unicode_argv():
-        prototype = ctypes.WINFUNCTYPE(ctypes.c_wchar_p)
-        GetCommandLineW = prototype(("GetCommandLineW",
-                                     ctypes.windll.kernel32))
-        command_line = GetCommandLineW()
-        if command_line is None:
-            raise ctypes.WinError()
-        # Skip the first argument, since we only care about parameters
-        argv = _command_line_to_argv(command_line, sys.argv)[1:]
-        return argv
-
-    def get_environ_unicode(key, default=None):
-        """Get `key` from environment as unicode or `default` if unset
-
-        The environment is natively unicode on modern windows versions but
-        Python 2 only accesses it through the legacy bytestring api.
-
-        Environmental variable names are case insenstive on Windows.
-
-        A large enough buffer will be allocated to retrieve the value, though
-        it may take two calls to the underlying library function.
-
-        This needs ctypes because pywin32 does not expose the wide version.
-        """
-        cfunc = getattr(get_environ_unicode, "_c_function", None)
-        if cfunc is None:
-            from ctypes.wintypes import DWORD, LPCWSTR, LPWSTR
-            cfunc = ctypes.WINFUNCTYPE(DWORD, LPCWSTR, LPWSTR, DWORD)(
-                ("GetEnvironmentVariableW", ctypes.windll.kernel32))
-            get_environ_unicode._c_function = cfunc
-        buffer_size = 256  # heuristic, 256 characters often enough
-        while True:
-            buffer = ctypes.create_unicode_buffer(buffer_size)
-            length = cfunc(key, buffer, buffer_size)
-            if not length:
-                code = ctypes.GetLastError()
-                if code == 203:  # ERROR_ENVVAR_NOT_FOUND
-                    return default
-                raise ctypes.WinError(code)
-            if buffer_size > length:
-                return buffer[:length]
-            buffer_size = length
-
-
-if has_win32api:
-    def _pywin32_is_local_pid_dead(pid):
-        """True if pid doesn't correspond to live process on this machine"""
-        try:
-            handle = win32api.OpenProcess(1, False, pid)  # PROCESS_TERMINATE
-        except pywintypes.error as e:
-            if e[0] == 5:  # ERROR_ACCESS_DENIED
-                # Probably something alive we're not allowed to kill
-                return False
-            elif e[0] == 87:  # ERROR_INVALID_PARAMETER
-                return True
-            raise
-        handle.close()
-        return False
-    is_local_pid_dead = _pywin32_is_local_pid_dead
-elif has_ctypes and sys.platform == 'win32':
-    from ctypes.wintypes import BOOL, DWORD, HANDLE
-    _kernel32 = ctypes.windll.kernel32
-    _CloseHandle = ctypes.WINFUNCTYPE(BOOL, HANDLE)(
-        ("CloseHandle", _kernel32))
-    _OpenProcess = ctypes.WINFUNCTYPE(HANDLE, DWORD, BOOL, DWORD)(
-        ("OpenProcess", _kernel32))
-
-    def _ctypes_is_local_pid_dead(pid):
-        """True if pid doesn't correspond to live process on this machine"""
-        handle = _OpenProcess(1, False, pid)  # PROCESS_TERMINATE
-        if not handle:
-            errorcode = ctypes.GetLastError()
-            if errorcode == 5:  # ERROR_ACCESS_DENIED
-                # Probably something alive we're not allowed to kill
-                return False
-            elif errorcode == 87:  # ERROR_INVALID_PARAMETER
-                return True
-            raise ctypes.WinError(errorcode)
-        _CloseHandle(handle)
-        return False
-    is_local_pid_dead = _ctypes_is_local_pid_dead
-
-
-def _is_pywintypes_error(evalue):
-    """True if exception instance is an error from pywin32"""
-    if has_pywintypes and isinstance(evalue, pywintypes.error):
-        return True
+def _ctypes_is_local_pid_dead(pid):
+    import ctypes
+    kernel32 = ctypes.wintypes.windll.kernel32
+    """True if pid doesn't correspond to live process on this machine"""
+    handle = kernel32.OpenProcess(1, False, pid)
+    if not handle:
+        errorcode = ctypes.GetLastError()
+        if errorcode == 5:  # ERROR_ACCESS_DENIED
+            # Probably something alive we're not allowed to kill
+            return False
+        elif errorcode == 87:  # ERROR_INVALID_PARAMETER
+            return True
+        raise ctypes.WinError(errorcode)
+    Kernel32.CloseHandle(handle)
     return False
+
+is_local_pid_dead = _ctypes_is_local_pid_dead
