@@ -27,8 +27,7 @@ To get a WorkingTree, call controldir.open_workingtree() or
 WorkingTree.open(dir).
 """
 
-from __future__ import absolute_import
-
+import contextlib
 import errno
 import os
 import sys
@@ -42,8 +41,6 @@ import stat
 
 from breezy import (
     conflicts as _mod_conflicts,
-    controldir,
-    errors,
     filters as _mod_filters,
     merge,
     revision as _mod_revision,
@@ -56,6 +53,16 @@ from breezy.bzr import (
     )
 """)
 
+from . import (
+    errors,
+    )
+from .controldir import (
+    ControlComponent,
+    ControlComponentFormatRegistry,
+    ControlComponentFormat,
+    ControlDir,
+    ControlDirFormat,
+    )
 from . import (
     osutils,
     )
@@ -65,9 +72,6 @@ from .symbol_versioning import deprecated_method, deprecated_in
 from .trace import mutter, note
 
 
-ERROR_PATH_NOT_FOUND = 3    # WindowsError errno code, equivalent to ENOENT
-
-
 class SettingFileIdUnsupported(errors.BzrError):
 
     _fmt = "This format does not support setting file ids."
@@ -78,7 +82,7 @@ class ShelvingUnsupported(errors.BzrError):
     _fmt = "This format does not support shelving changes."
 
 
-class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
+class WorkingTree(mutabletree.MutableTree, ControlComponent):
     """Working copy tree.
 
     :ivar basedir: The root of the tree on disk. This is a unicode path object
@@ -123,6 +127,9 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
     def control_transport(self):
         return self._transport
 
+    def supports_symlinks(self):
+        return osutils.supports_symlinks(self.basedir)
+
     def is_control_filename(self, filename):
         """True if filename is the name of a control file in this tree.
 
@@ -153,10 +160,7 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
         return self._format.supports_merge_modified
 
     def _supports_executable(self):
-        if sys.platform == 'win32':
-            return False
-        # FIXME: Ideally this should check the file system
-        return True
+        return osutils.supports_executable(self.basedir)
 
     def break_lock(self):
         """Break a lock if one is present from another instance.
@@ -198,7 +202,7 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
         """
         if path is None:
             path = osutils.getcwd()
-        control = controldir.ControlDir.open(path, _unsupported=_unsupported)
+        control = ControlDir.open(path, _unsupported=_unsupported)
         return control.open_workingtree(unsupported=_unsupported)
 
     @staticmethod
@@ -216,7 +220,7 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
         """
         if path is None:
             path = osutils.getcwd()
-        control, relpath = controldir.ControlDir.open_containing(path)
+        control, relpath = ControlDir.open_containing(path)
         return control.open_workingtree(), relpath
 
     @staticmethod
@@ -294,24 +298,6 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
         """
         return WorkingTree.open(path, _unsupported=True)
 
-    @staticmethod
-    def find_trees(location):
-        def list_current(transport):
-            return [d for d in transport.list_dir('')
-                    if not controldir.is_control_filename(d)]
-
-        def evaluate(controldir):
-            try:
-                tree = controldir.open_workingtree()
-            except errors.NoWorkingTree:
-                return True, None
-            else:
-                return True, tree
-        t = transport.get_transport(location)
-        iterator = controldir.ControlDir.find_controldirs(t, evaluate=evaluate,
-                                                          list_current=list_current)
-        return [tr for tr in iterator if tr is not None]
-
     def __repr__(self):
         return "<%s of %s>" % (self.__class__.__name__,
                                getattr(self, 'basedir', None))
@@ -380,7 +366,11 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
         stat_value = _fstat(file_obj.fileno())
         if filtered and self.supports_content_filtering():
             filters = self._content_filter_stack(path)
-            file_obj = _mod_filters.filtered_input_file(file_obj, filters)
+            if filters:
+                file_obj, size = _mod_filters.filtered_input_file(
+                    file_obj, filters)
+                stat_value = _mod_filters.FilteredStat(
+                    stat_value, st_size=size)
         return (file_obj, stat_value)
 
     def get_file_text(self, path, filtered=True):
@@ -413,10 +403,6 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
                 parents.append(revision_id)
         return parents
 
-    def get_root_id(self):
-        """Return the id of this trees root"""
-        raise NotImplementedError(self.get_root_id)
-
     def clone(self, to_controldir, revision_id=None):
         """Duplicate this working tree into to_bzr, including all state.
 
@@ -439,7 +425,7 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
     def copy_content_into(self, tree, revision_id=None):
         """Copy the current content and user files of this tree into tree."""
         with self.lock_read():
-            tree.set_root_id(self.get_root_id())
+            tree.set_root_id(self.path2id(''))
             if revision_id is None:
                 merge.transform_tree(tree, self)
             else:
@@ -673,7 +659,7 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
             merger.other_rev_id = to_revision
             if _mod_revision.is_null(merger.other_rev_id):
                 raise errors.NoCommits(branch)
-            self.branch.fetch(branch, last_revision=merger.other_rev_id)
+            self.branch.fetch(branch, stop_revision=merger.other_rev_id)
             merger.other_basis = merger.other_rev_id
             merger.other_tree = self.branch.repository.revision_tree(
                 merger.other_rev_id)
@@ -723,37 +709,18 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
 
     def get_symlink_target(self, path):
         abspath = self.abspath(path)
-        target = osutils.readlink(abspath)
-        return target
+        try:
+            return osutils.readlink(abspath)
+        except OSError as e:
+            if getattr(e, 'errno', None) == errno.ENOENT:
+                raise errors.NoSuchFile(path)
+            raise
 
     def subsume(self, other_tree):
         raise NotImplementedError(self.subsume)
 
-    def _setup_directory_is_tree_reference(self):
-        if self._branch.repository._format.supports_tree_reference:
-            self._directory_is_tree_reference = \
-                self._directory_may_be_tree_reference
-        else:
-            self._directory_is_tree_reference = \
-                self._directory_is_never_tree_reference
-
-    def _directory_is_never_tree_reference(self, relpath):
-        return False
-
-    def _directory_may_be_tree_reference(self, relpath):
-        # as a special case, if a directory contains control files then
-        # it's a tree reference, except that the root of the tree is not
-        return relpath and osutils.isdir(self.abspath(relpath) + u"/.bzr")
-        # TODO: We could ask all the control formats whether they
-        # recognize this directory, but at the moment there's no cheap api
-        # to do that.  Since we probably can only nest bzr checkouts and
-        # they always use this name it's ok for now.  -- mbp 20060306
-        #
-        # FIXME: There is an unhandled case here of a subdirectory
-        # containing .bzr but not a branch; that will probably blow up
-        # when you try to commit it.  It might happen if there is a
-        # checkout in a subdirectory.  This can be avoided by not adding
-        # it.  mbp 20070306
+    def _directory_is_tree_reference(self, relpath):
+        raise NotImplementedError(self._directory_is_tree_reference)
 
     def extract(self, path, format=None):
         """Extract a subtree from this tree.
@@ -769,7 +736,8 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
     def kind(self, relpath):
         return osutils.file_kind(self.abspath(relpath))
 
-    def list_files(self, include_root=False, from_dir=None, recursive=True):
+    def list_files(self, include_root=False, from_dir=None, recursive=True,
+                   recurse_nested=False):
         """List all files as (path, class, kind, id, entry).
 
         Lists, but does not descend into unversioned directories.
@@ -855,13 +823,13 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
 
     def pull(self, source, overwrite=False, stop_revision=None,
              change_reporter=None, possible_transports=None, local=False,
-             show_base=False):
+             show_base=False, tag_selector=None):
         with self.lock_write(), source.lock_read():
             old_revision_info = self.branch.last_revision_info()
             basis_tree = self.basis_tree()
             count = self.branch.pull(source, overwrite, stop_revision,
                                      possible_transports=possible_transports,
-                                     local=local)
+                                     local=local, tag_selector=tag_selector)
             new_revision_info = self.branch.last_revision_info()
             if new_revision_info != old_revision_info:
                 repository = self.branch.repository
@@ -879,8 +847,8 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
                         this_tree=self,
                         change_reporter=change_reporter,
                         show_base=show_base)
-                    basis_root_id = basis_tree.get_root_id()
-                    new_root_id = new_basis_tree.get_root_id()
+                    basis_root_id = basis_tree.path2id('')
+                    new_root_id = new_basis_tree.path2id('')
                     if new_root_id is not None and basis_root_id != new_root_id:
                         self.set_root_id(new_root_id)
                 # TODO - dedup parents list with things merged by pull ?
@@ -1036,31 +1004,28 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
     def revert(self, filenames=None, old_tree=None, backups=True,
                pb=None, report_changes=False):
         from .conflicts import resolve
-        with self.lock_tree_write():
+        with contextlib.ExitStack() as exit_stack:
+            exit_stack.enter_context(self.lock_tree_write())
             if old_tree is None:
                 basis_tree = self.basis_tree()
-                basis_tree.lock_read()
+                exit_stack.enter_context(basis_tree.lock_read())
                 old_tree = basis_tree
             else:
                 basis_tree = None
-            try:
-                conflicts = transform.revert(self, old_tree, filenames, backups, pb,
-                                             report_changes)
-                if filenames is None and len(self.get_parent_ids()) > 1:
-                    parent_trees = []
-                    last_revision = self.last_revision()
-                    if last_revision != _mod_revision.NULL_REVISION:
-                        if basis_tree is None:
-                            basis_tree = self.basis_tree()
-                            basis_tree.lock_read()
-                        parent_trees.append((last_revision, basis_tree))
-                    self.set_parent_trees(parent_trees)
-                    resolve(self)
-                else:
-                    resolve(self, filenames, ignore_misses=True, recursive=True)
-            finally:
-                if basis_tree is not None:
-                    basis_tree.unlock()
+            conflicts = transform.revert(self, old_tree, filenames, backups, pb,
+                                         report_changes)
+            if filenames is None and len(self.get_parent_ids()) > 1:
+                parent_trees = []
+                last_revision = self.last_revision()
+                if last_revision != _mod_revision.NULL_REVISION:
+                    if basis_tree is None:
+                        basis_tree = self.basis_tree()
+                        exit_stack.enter_context(basis_tree.lock_read())
+                    parent_trees.append((last_revision, basis_tree))
+                self.set_parent_trees(parent_trees)
+                resolve(self)
+            else:
+                resolve(self, filenames, ignore_misses=True, recursive=True)
             return conflicts
 
     def store_uncommitted(self):
@@ -1088,7 +1053,6 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
             if file_id is None:
                 raise ValueError(
                     'WorkingTree.set_root_id with fileid=None')
-            file_id = osutils.safe_file_id(file_id)
             self._set_root_id(file_id)
 
     def _set_root_id(self, file_id):
@@ -1111,10 +1075,8 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
         """
         raise NotImplementedError(self.unlock)
 
-    _marker = object()
-
     def update(self, change_reporter=None, possible_transports=None,
-               revision=None, old_tip=_marker, show_base=False):
+               revision=None, old_tip=None, show_base=False):
         """Update a working tree along its branch.
 
         This will update the branch if its bound too, which means we have
@@ -1146,105 +1108,7 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
             returned (old tip of the branch or None). _marker is used
             otherwise.
         """
-        if self.branch.get_bound_location() is not None:
-            self.lock_write()
-            update_branch = (old_tip is self._marker)
-        else:
-            self.lock_tree_write()
-            update_branch = False
-        try:
-            if update_branch:
-                old_tip = self.branch.update(possible_transports)
-            else:
-                if old_tip is self._marker:
-                    old_tip = None
-            return self._update_tree(old_tip, change_reporter, revision, show_base)
-        finally:
-            self.unlock()
-
-    def _update_tree(self, old_tip=None, change_reporter=None, revision=None,
-                     show_base=False):
-        """Update a tree to the master branch.
-
-        :param old_tip: if supplied, the previous tip revision the branch,
-            before it was changed to the master branch's tip.
-        """
-        # here if old_tip is not None, it is the old tip of the branch before
-        # it was updated from the master branch. This should become a pending
-        # merge in the working tree to preserve the user existing work.  we
-        # cant set that until we update the working trees last revision to be
-        # one from the new branch, because it will just get absorbed by the
-        # parent de-duplication logic.
-        #
-        # We MUST save it even if an error occurs, because otherwise the users
-        # local work is unreferenced and will appear to have been lost.
-        #
-        with self.lock_tree_write():
-            nb_conflicts = 0
-            try:
-                last_rev = self.get_parent_ids()[0]
-            except IndexError:
-                last_rev = _mod_revision.NULL_REVISION
-            if revision is None:
-                revision = self.branch.last_revision()
-
-            old_tip = old_tip or _mod_revision.NULL_REVISION
-
-            if not _mod_revision.is_null(old_tip) and old_tip != last_rev:
-                # the branch we are bound to was updated
-                # merge those changes in first
-                base_tree = self.basis_tree()
-                other_tree = self.branch.repository.revision_tree(old_tip)
-                nb_conflicts = merge.merge_inner(self.branch, other_tree,
-                                                 base_tree, this_tree=self,
-                                                 change_reporter=change_reporter,
-                                                 show_base=show_base)
-                if nb_conflicts:
-                    self.add_parent_tree((old_tip, other_tree))
-                    note(gettext('Rerun update after fixing the conflicts.'))
-                    return nb_conflicts
-
-            if last_rev != _mod_revision.ensure_null(revision):
-                # the working tree is up to date with the branch
-                # we can merge the specified revision from master
-                to_tree = self.branch.repository.revision_tree(revision)
-                to_root_id = to_tree.get_root_id()
-
-                basis = self.basis_tree()
-                with basis.lock_read():
-                    if (basis.get_root_id() is None or basis.get_root_id() != to_root_id):
-                        self.set_root_id(to_root_id)
-                        self.flush()
-
-                # determine the branch point
-                graph = self.branch.repository.get_graph()
-                base_rev_id = graph.find_unique_lca(self.branch.last_revision(),
-                                                    last_rev)
-                base_tree = self.branch.repository.revision_tree(base_rev_id)
-
-                nb_conflicts = merge.merge_inner(self.branch, to_tree, base_tree,
-                                                 this_tree=self,
-                                                 change_reporter=change_reporter,
-                                                 show_base=show_base)
-                self.set_last_revision(revision)
-                # TODO - dedup parents list with things merged by pull ?
-                # reuse the tree we've updated to to set the basis:
-                parent_trees = [(revision, to_tree)]
-                merges = self.get_parent_ids()[1:]
-                # Ideally we ask the tree for the trees here, that way the working
-                # tree can decide whether to give us the entire tree or give us a
-                # lazy initialised tree. dirstate for instance will have the trees
-                # in ram already, whereas a last-revision + basis-inventory tree
-                # will not, but also does not need them when setting parents.
-                for parent in merges:
-                    parent_trees.append(
-                        (parent, self.branch.repository.revision_tree(parent)))
-                if not _mod_revision.is_null(old_tip):
-                    parent_trees.append(
-                        (old_tip, self.branch.repository.revision_tree(old_tip)))
-                self.set_parent_trees(parent_trees)
-                last_rev = parent_trees[0][0]
-            return nb_conflicts
+        raise NotImplementedError(self.update)
 
     def set_conflicts(self, arg):
         raise errors.UnsupportedOperation(self.set_conflicts, self)
@@ -1259,8 +1123,8 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
         """Walk the directories of this tree.
 
         returns a generator which yields items in the form:
-                ((curren_directory_path, fileid),
-                 [(file1_path, file1_name, file1_kind, (lstat), file1_id,
+                (current_directory_path,
+                 [(file1_path, file1_name, file1_kind, (lstat),
                    file1_kind), ... ])
 
         This API returns a generator, which is only valid during the current
@@ -1280,11 +1144,11 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
         into files that have text conflicts.  The corresponding .THIS .BASE and
         .OTHER files are deleted, as per 'resolve'.
 
-        :return: a tuple of ConflictLists: (un_resolved, resolved).
+        :return: a tuple of lists: (un_resolved, resolved).
         """
         with self.lock_tree_write():
-            un_resolved = _mod_conflicts.ConflictList()
-            resolved = _mod_conflicts.ConflictList()
+            un_resolved = []
+            resolved = []
             for conflict in self.conflicts():
                 try:
                     conflict.action_auto(self)
@@ -1366,8 +1230,17 @@ class WorkingTree(mutabletree.MutableTree, controldir.ControlComponent):
         with self.lock_read():
             return next(self.get_canonical_paths([path]))
 
+    def reference_parent(self, path, branch=None, possible_transports=None):
+        raise errors.UnsupportedOperation(self.reference_parent, self)
 
-class WorkingTreeFormatRegistry(controldir.ControlComponentFormatRegistry):
+    def get_reference_info(self, path, branch=None):
+        raise errors.UnsupportedOperation(self.get_reference_info, self)
+
+    def set_reference_info(self, tree_path, branch_location):
+        raise errors.UnsupportedOperation(self.set_reference_info, self)
+
+
+class WorkingTreeFormatRegistry(ControlComponentFormatRegistry):
     """Registry for working tree formats."""
 
     def __init__(self, other_registry=None):
@@ -1396,7 +1269,7 @@ class WorkingTreeFormatRegistry(controldir.ControlComponentFormatRegistry):
 format_registry = WorkingTreeFormatRegistry()
 
 
-class WorkingTreeFormat(controldir.ControlComponentFormat):
+class WorkingTreeFormat(ControlComponentFormat):
     """An encapsulation of the initialization and open routines for a format.
 
     Formats provide three things:
